% a demonstration of several HU algorithms

addpath sunsal
addpath sisal

% both sisal and sunsal are from Prof. Jose Bioucas-Dias (http://www.lx.it.pt/~bioucas)
% Sisal: A Variable splitting Augmented Lagragian Approach to Linear Spectral Unmixing
%         J. Bioucas-Dias, "A variable splitting augmented Lagrangian approach to linear spectral unmixing", in First IEEE GRSS Workshop on Hyperspectral Image and Signal Processing-WHISPERS'2009, Grenoble, France, 2009. Available at http://arxiv.org/abs/0904.4635v
% Sunsal: http://www.lx.it.pt/~bioucas/code/sunsal_demo.zip


Abstract

clear; clc; close all;

%% data generation
M = 50;    % Number of bands
N = 3;     % Number of endmembers
L = 1000;  % Number of pixels

SOR = -10;

A = rand(M,N);
%------- generate the endmembers -----



SNR = 18;
L_out = 20; % number of outliers
SHAPE_PARAMETER = 1;      %(Dirichlet parameter) abundances uniformely
%distribted over the simplex
MAX_PURIRY = .8;           % do not theshold abundances
OUTLIERS   = 0;           % no outliers
PURE_PIXELS = 'no';       % no pure pixels

% generae the data (the function is from Prof. Jose Bioucas-Dias)
[y,s] = spectMixGen(A,L-L_out, ...
    'Source_pdf', 'Diri_id', ...
    'pdf_pars',SHAPE_PARAMETER,...
    'max_purity',MAX_PURIRY*ones(1,N), ...
    'no_outliers',OUTLIERS, ...
    'pure_pixels', PURE_PIXELS, ...
    'violation_extremes',[1,1.2], ....
    'snr', SNR, ...
    'noise_shape','uniform');

% adding outliers

O = rand(M,L_out);
SigEngy = sum(sum((A*s).^2))/L;
OutEngy = SigEngy/(10^(SOR/10));
O = (O/sqrt(sum(sum(O.^2))))*sqrt(L_out*OutEngy);
SOR_check = 10*log10((sum(sum((A*s).^2))/L)/(sum(sum(O.^2))/L_out));

y = [y,O];
        
y_clean = y(:,1:end-L_out); % for visualziation
     
%--------------- ASF and plot--------



d = mean(y_clean,2);
U = y_clean-d*ones(1,L-L_out);
OPTS.disp = 0;
[C D] = eigs(U*U',N-1,'LM',OPTS);
y_r = C'*(y-d*ones(1,L));  % supposed to be a 2-D data 


%---groud truth of projected A 
A_r =C'*(A-d*ones(1,N));

figure(1)
plot(y_r(1,:),y_r(2,:),'b*');
hold on
h0=plot(A_r(1,:),A_r(2,:),'ro','linewidth',2,'markersize',16); hold on

%------------------------------------------------------------------
%                               Algorithms
%------------------------------------------------------------------
 
y_sisal = y; 

[A_sisal] = sisal(y_sisal,N,'tau',1);
A_sisal_r =C'*(A_sisal-d*ones(1,N));

figure(1) 
h1=plot(A_sisal_r(1,:),A_sisal_r(2,:),'rs','linewidth',2); hold on
% if you want to plot the facets of the estimated simplex
A_c = [A_sisal_r,A_sisal_r(:,1)];
plot(A_c(1,:), A_c(2,:),'--r','linewidth',2)



 
%% 
[A_sisal] = sisal(y_sisal,N,'tau',.1);
A_sisal_r =C'*(A_sisal-d*ones(1,N));

figure(1)
h11=plot(A_sisal_r(1,:),A_sisal_r(2,:),'r+','linewidth',2); hold on
% if you want to plot the facets of the estimated simplex
A_c = [A_sisal_r,A_sisal_r(:,1)]; 
plot(A_c(1,:), A_c(2,:),'--r','linewidth',2) 




%% RVolMin

% compute S for initialization
[S_sisal,res_p,res_d] = sunsal(A_sisal, y, 'lambda', 0 , 'POSITIVITY','yes', 'ADDONE', 'yes','verbose' ,'no');  
   

p = .5;    
lambda = 1;
[ A_rvolmin,S_rvolmin,]  = RVolMin( y,N,'p',.5,'lambda',lambda,'MaxIt',5000,'use_ini','yes','A_ini',A_sisal,'S_ini',S_sisal,'solver_prox','FISTA','nonneg_A','no');
 
A_rvolmin_r =C'*(A_rvolmin-d*ones(1,N));
  
       
figure(1)
h33=plot(A_rvolmin_r(1,:),A_rvolmin_r(2,:),'bd','linewidth',2,'markersize',12); hold on
A_c = [A_rvolmin_r,A_rvolmin_r(:,1)];
plot(A_c(1,:), A_c(2,:),'--b','linewidth',2) 


legend([h0 h1 h11 h33],'groudtruth','SISAL (\eta =1)','SISAL (\eta = .1)','Proposed')
set(gca,'Xtick',[]) 
set(gca,'Ytick',[])



